#include "cc3000\cc3000_patch.h"
#include "cc3000\cc3000.h"
#include "cc3000\cc3000_hci.h"
#include "cc3000\cc3000_nvmem.h"
#include "pic_utils.h"
#include "memory.h"
#include "pic_serial.h"

const unsigned char cRMdefaultParams[128] = { 0x03, 0x00, 0x01, 0x01, 0x14, 0x14, 0x00, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x23, 0x25, 0x25, 0x25, 0x25, 0x25, 0x25, 0x25, 0x25, 0x25, 0x23, 0x23, 0x23, 0x23, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50, 0x01, 0x77, 0x80, 0x1D, 0x1F, 0x22, 0x26, 0x28, 0x29, 0x1A, 0x1F, 0x22, 0x24, 0x26, 0x28, 0x16, 0x1D, 0x1E, 0x20, 0x24, 0x25, 0x1E, 0x2D, 0x01, 0x02, 0x02, 0x02, 0x02, 0x00, 0x15, 0x15, 0x15, 0x11, 0x15, 0x15, 0x0E, 0x00};

// array to store RM parameters from EEPROM
unsigned char cRMParamsFromEeprom[128];

// array to store MAC address from EEPROM
unsigned char cMacFromEeprom[MAC_ADDR_LEN];

// 2 dim array to store address and length of new FAT
const unsigned short aFATEntries[2][NVMEM_RM_FILEID + 1] = 
/*  address 	*/  {{0x50, 	0x1f0, 	0x390, 	0x1390, 	0x2390, 	0x4390, 	0x6390, 	0x63a0, 	0x63b0, 	0x63f0, 	0x6430, 	0x6830},
/*  length	*/	{0x1a0, 	0x1a0, 	0x1000, 	0x1000, 	0x2000, 	0x2000, 	0x10, 	0x10, 	0x40, 	0x40, 	0x400, 	0x200}};
/* 0. NVS */
/* 1. NVS Shadow */
/* 2. Wireless Conf */
/* 3. Wireless Conf Shadow */
/* 4. BT (WLAN driver) Patches */
/* 5. WiLink (Firmware) Patches */
/* 6. MAC addr */
/* 7. Frontend Vars */
/* 8. IP config */
/* 9. IP config Shadow */
/* 10. Bootloader Patches */
/* 11. Radio Module params */
/* 12. AES128 for smart config */
/* 13. user file */
/* 14. user file */
/* 15. user file */

	unsigned char   *pRMParams;
	unsigned short  index;

//*****************************************************************************
//
//! fat_write_content
//!
//! \param[in] file_address  array of file address in FAT table:\n
//!						 this is the absolute address of the file in the EEPROM.
//! \param[in] file_length  array of file length in FAT table:\n
//!						 this is the upper limit of the file size in the EEPROM.
//!
//! \return on succes 0, error otherwise
//!
//! \brief  parse the FAT table from eeprom 
//
//*****************************************************************************
uns8 cc3000_patch_fat_write_content(uns16 *file_address, uns16 *file_length) {
	uns16  index = 0;
	uns8   ucStatus;
	uns8   fatTable[48];
	uns8*  fatTablePtr = fatTable;
	
	// first, write the magic number
	ucStatus = cc3000_nvmem_write(16, 2, 0, "LS"); 
	
	for (; index <= NVMEM_RM_FILEID; index++)
	{
		// write address low char and mark as allocated
		*fatTablePtr++ = (unsigned char)(file_address[index] & 0xff) | 0b00000001; // BIT0;
		
		// write address high char
		*fatTablePtr++ = (unsigned char)((file_address[index]>>8) & 0xff);
		
		// write length low char
		*fatTablePtr++ = (unsigned char)(file_length[index] & 0xff);
		
		// write length high char
		*fatTablePtr++ = (unsigned char)((file_length[index]>>8) & 0xff);		
	}
	
	// second, write the FAT
	// write in two parts to work with tiny driver
	ucStatus = cc3000_nvmem_write(16, 24, 4, fatTable); 
	ucStatus = cc3000_nvmem_write(16, 24, 24+4, &fatTable[24]); 
	
	// third, we want to erase any user files
	memset(fatTable, 0, sizeof(fatTable));
	ucStatus = cc3000_nvmem_write(16, 16, 52, fatTable); 
	
	return ucStatus;
}


hci_status_t mac_status;

unsigned char ucStatus_Dr, ucStatus_FW, return_status = 0xFF;
//signed char mac_status = -1;
unsigned char counter = 0;


void cc3000_patch_store_data() {

	// read MAC address
	debug_str("Getting Mac address\n");
	mac_status = cc3000_nvmem_get_mac_address(cMacFromEeprom);

	serial_print_str("\nGot mac: ");
	for (uns8 count = 0; count < 6; count++) {
		serial_print_int_hex(cMacFromEeprom[count]);
		if (count !=5) {
			serial_putc(':');
		}
	}
	serial_print_nl();

	debug_str("Now getting radio params\n");
	
	return_status = 1;
	
	while ((return_status) && (counter < 3))
	{
		
		return_status = 0;
		pRMParams = cRMParamsFromEeprom;
		
		for (index = 0; index < 16; index++)
		{
			return_status |= cc3000_nvmem_read(NVMEM_RM_FILEID, 8, 8*index, pRMParams); 
			pRMParams += 8;
		}
		counter++;
	}
	
	// if RM file is not valid, load the default one
	if (counter == 3)
	{
			
		debug_str("No good, using defaults\n");
		pRMParams = (unsigned char *)cRMdefaultParams;
	}
	else
	{
		debug_str("Params look good\n");
		pRMParams = cRMParamsFromEeprom;
	}
	
	return_status = 1;
}

void cc3000_patch_fix_fat() {
	
	debug_str("Time to fix the fat (scary)\n");
		
	while (return_status)
	{
		// write new FAT
		return_status = cc3000_patch_fat_write_content(aFATEntries[0], aFATEntries[1]);
	}
	
	//wlan_stop();
	cc3000_shutdown();
	//initDriver(1);
	cc3000_startup(1);
}

void cc3000_patch_restore_data() {	
	return_status = 1;
	
	while (return_status)
	{
		// write RM parameters
		// write in 4 parts to work with tiny driver
		
		return_status = 0;
		
		for (index = 0; index < 4; index++)
		{
			return_status |= cc3000_nvmem_write(NVMEM_RM_FILEID, 32, 32*index, (pRMParams + 32*index)); 
		}
	}
	
	return_status = 1;
	
	// write back the MAC address, only if exist
	if (mac_status == 0)
	{
		// zero out MCAST bit if set
		cMacFromEeprom[0] &= 0xfe;
		while (return_status)
		{
			return_status = cc3000_nvmem_set_mac_address(cMacFromEeprom);
		}
	}
	
}

// convert to internal flash format

hci_status_t cc3000_nvmem_write_patch(uns8 file_id, uns16 length, uns16 mem_location);
{
	uns8	status = 0;
	uns16	offset = 0;

	uns8 data[32];

	tblptrl = mem_location & 0xff;
	tblptrh = mem_location >> 8;


	
	while ((status == 0) && (length >= 32))
	{
		
		// Fill array from flash
		
		for (uns8 count = 0; count < 32; count++) {
			asm TBLRD*+
			data[count] = tablat;
		}	
		
		// write it out
		
		status = nvmem_write(ulFileId, 32, offset, &data);
		offset += 32;
		spLength -= 32;
		spDataPtr += 32;
	}
	
	if (status !=0)
	{
		// NVMEM error occurred
		debug_str("!!!---- Error writing flash!\n");
		return status;
	}
	
	if (length != 0)
	{
		// if reached here, a reminder is left
		for (uns8 count = 0; count < length; count++) {
			asm TBLRD*+
			data[count] = tablat;
		}	
		status = nvmem_write(ulFileId, length, offset, &data);
	}
	
	return status;
}

void cc3000_patch_update_dr() {

ucStatus_Dr = 1;
	
	uns16 patch_loc = dr_patch_get_data_loc()
	
	debug_str("Update patch. Mem loc = ");
	debug_int_hex_16bit(patch_loc);
	
	while (ucStatus_Dr)
	{
		ucStatus_Dr = cc3000_nvmem_write_patch(NVMEM_WLAN_DRIVER_SP_FILEID, DR_PATCHES_SIZE, patch_loc);        
	}

}

void cc3000_patch_update_fw() {

ucStatus_FW = 1;

	uns16 patch_loc = fw_patch_get_data_loc()
	
	debug_str("Update patch. Mem loc = ");
	debug_int_hex_16bit(patch_loc);

	while (ucStatus_FW)
	{
		ucStatus_FW = nvmem_write_patch(NVMEM_WLAN_FW_SP_FILEID, FW_PATCHES_SIZE, patch_loc);
	}
}	

void wait_enter_key() {

uns8 key;

	key = 0;
	while ((!serial_rx_avail()) && (key != 13)) {
		key = serial_getc();
	}

}	

void cc3000_patch_execute() {

uns8 sp_maj_before, sp_min_before, sp_maj_after, sp_min_after;
uns8 status;


	debug_str("Ready for patch update. <enter>:");
	wait_enter_key();
	debug_nl();
	
	debug_str("---- INIT DRIVER (1): <enter>");
	wait_enter_key();
	debug_nl();
	
	cc3000_startup(1);
	
			status = cc3000_nvmem_read_sp_version(&sp_maj_before, &sp_min_before);
			debug_var("\nstatus=", status);
			serial_print_str(" service pack = ");
			serial_print_int(sp_maj_before);
			serial_putc('.');
			serial_print_int(sp_min_before);
			serial_print_nl();

	debug_str("---- STORE DATA : <enter>");
	wait_enter_key();
	debug_nl();
	
	cc3000_patch_store_data();
	
	debug_str("---- Fix FAT : <enter>");
	wait_enter_key();
	debug_nl();	
	cc3000_patch_fix_fat();
	
	debug_str("---- Restore data : <enter>");
	wait_enter_key();
	debug_nl();

	cc3000_restore_data();
	
	debug_str("---- Write driver patch : <enter>");
	wait_enter_key();
	debug_nl();
	cc3000_patch_update_dr();
	
	debug_str("---- Write fw patch : <enter>");
	wait_enter_key();
	debug_nl();
	cc3000_patch_update_fw();
	
	debug_str("---- Reboot module <enter>");
	wait_enter_key();
	debug_nl();
	cc3000_shutdown();
	delay_ms(250);
	cc3000_startup(0);

			status = cc3000_nvmem_read_sp_version(&sp_maj_after, &sp_min_after);
			debug_var("\nstatus=", status);
			serial_print_str(" Now - service pack = ");
			serial_print_int(sp_maj_after);
			serial_putc('.');
			serial_print_int(sp_min_after);
			serial_print_nl();
			serial_print_str(" (Previously ");
			serial_print_int(sp_maj_before);
			serial_putc('.');
			serial_print_int(sp_min_before);
			serial_print_str(")\n");
		
}
	
			
	