/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/


/*! \file pic_term.h
    \brief Pic terminal routines

	Allows the use of a serial terminal for interaction with the PIC
 

Put the following into your config.h
\code
// - - - - - - - - - - - - - - - - - - - - 
// pic_term defines
// - - - - - - - - - - - - - - - - - - - - 

#define TERM_BUFFER_SIZE	10

// Reset (go to bootloader) if magic character received
#define TERM_ALLOW_BOOSTBLOADER
// Echo typing so user can see what they're doing
#define TERM_ECHO_INPUT

// Use USB instead of serial
#define TERM_USE_USB

// - - - - - - - - - - - - - - - - - - - - 

// Create term_entry_callback in your own code
// void term_entry_callback(uns8 *term_buffer);
\endcode
*/

#ifndef __pic_term_h
#define __pic_term_h

#include "pic_utils.h"
#include "config.h"
#ifndef TERM_USE_USB
	#include "pic_serial.h"
#endif	

void term_set_terminator(uns8 terminator);

void term_process();
extern void term_entry_callback(uns8 *term_buffer);
void term_init();



#endif